@@ -4,17 +4,19 @@
 # USAGE: ". $SPECC/bin/setup.sh"
 #
 # author:	Rainer Doemer
-# last update:	12/28/01
+# last update:	June 24, 2004
 
 
 # define the SpecC system home directory
-export SPECC; SPECC=/home/rainer/project/SpecC/scrc-1.2/install
+export SPECC; SPECC=/home/specc/scrc/current/installed
 
 # extend the search path
 export PATH; PATH=$PATH:$SPECC/bin
 
-# extend the manual path
-export MANPATH; MANPATH=$MANPATH:$SPECC/man
+# extend the manual path (if any)
+if [ -n "$MANPATH" ]; then
+	export MANPATH; MANPATH=$MANPATH:$SPECC/man
+fi
 
 # extend the search path for dynamically-linked libraries
 # (usually not necessary)
