@@ -2,6 +2,8 @@
 // bit.h: Bit Vector Library, user level
 // ============================================================================
 // 
+// 11/28/01 <gerstl>	Removed constructors from shadow _longlong_
+// 08/10/01 <gerstl>	Introduced len_t, bnd_t; changed shift cnt to unsigned
 // 05/24/01 <gerstl>	Initial version
 //
 // ----------------------------------------------------------------------------
@@ -12,17 +14,19 @@
 #include <sys/bit.h>
 
 
-
 /* ------------------------------------------------------------------------ */
 /* Bit Vectors */
 
 
+// Forward declarations
+template<_bit::len_t,bool> class _bit_;
+
 
 /* --- Bit vector representation --- */
 
-template<int len> class _bitrep
+template<_bit::len_t len> class _bitrep
 {
-protected:
+public:
   // Array of chunks
   _bit::chunk  vec[_BITCHUNKS(len)];
 };
@@ -30,14 +34,18 @@ protected:
 
 /* --- Bit vector class --- */
 
-template<int len, bool usign=false> class bit: public _bitrep<len>
+template<_bit::len_t len, bool usign=false> class bit: protected _bitrep<len>
 {
+  typedef _bit::len_t len_t;  
+  typedef _bit::bnd_t bnd_t;
+  
 public:
   // Constructors
   bit()                                                                     { }
   bit(const _bit& c)                                             { ref() = c; }
   bit(const _bitslice& s)                                        { ref() = s; }
-  template<int l, bool us> bit(const bit<l,us>& c)               { ref() = c; }
+  template<len_t l, bool us> bit(const bit<l,us>& c)             { ref() = c; }
+  template<len_t l, bool us> bit(const _bit_<l,us>& c)           { ref() = c; }
   bit(const char* s)                                             { ref() = s; }
   bit(int i)                                                     { ref() = i; }
   bit(unsigned int i)                                            { ref() = i; }
@@ -49,8 +57,10 @@ public:
   // Assignments
   bit& operator=(const _bit& o)                    { ref() = o; return put(); }
   bit& operator=(const _bitslice& o)               { ref() = o; return put(); }
-  template<int l, bool us> 
+  template<len_t l, bool us> 
     bit& operator=(const bit<l,us>& c)             { ref() = c; return put(); }
+  template<len_t l, bool us> 
+    bit& operator=(const _bit_<l,us>& c)           { ref() = c; return put(); }
   bit& operator=(const char* s)                    { ref() = s; return put(); }
   bit& operator=(int i)                            { ref() = i; return put(); }
   bit& operator=(unsigned int i)                   { ref() = i; return put(); }
@@ -74,8 +84,8 @@ public:
   bit& operator%=(const _bit& op)         { get(); ref() %= op; return put(); }
 
   // Shift operators
-  bit& operator<<=(int sh)               { get(); ref() <<= sh; return put(); }
-  bit& operator>>=(int sh)               { get(); ref() >>= sh; return put(); }
+  bit& operator<<=(unsigned int sh)      { get(); ref() <<= sh; return put(); }
+  bit& operator>>=(unsigned int sh)      { get(); ref() >>= sh; return put(); }
 
   // Flip the bits in the vector
   bit& flip(void)                        { get(); ref().flip(); return put(); }
@@ -85,7 +95,7 @@ public:
   bit& operator-- ()                        { get(); --(ref()); return put(); }
   bit  operator++ (int)           { bit r(get()); ++(ref()); put(); return r; }
   bit  operator-- (int)           { bit r(get()); --(ref()); put(); return r; }
-  
+
   // Test for != 0
   bool test(void) const                                { return val().test(); }
 
@@ -100,41 +110,41 @@ public:
   long double toLDouble(void) const               { return val().toLDouble(); }
                                   
   // Slicing
-  _bitslice operator[](int i)                     { return slice(i, i, true); }
-  _bitslice operator()(int sl, int sr)                { return slice(sl, sr); }
-  const _bitslice operator[](int i) const         { return slice(i, i, true); }
-  const _bitslice operator()(int sl, int sr) const    { return slice(sl, sr); }
+  _bitslice operator[](len_t i)                   { return slice(i, i, true); }
+  _bitslice operator()(len_t sl, len_t sr)            { return slice(sl, sr); }
+  const _bitslice operator[](len_t i) const       { return slice(i, i, true); }
+  const _bitslice operator()(len_t sl, len_t sr) const {return slice(sl, sr); }
 
   // Mapped slicing 
-  _bitslice operator()(int l, int r, int i)    
+  _bitslice operator()(bnd_t l, bnd_t r, bnd_t i)    
               { ASSERT(_BITLEN(l,r)==len); return operator[](_BITMAP(l,r,i)); }
-  _bitslice operator()(int l, int r, int sl, int sr) 
+  _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) 
     {ASSERT(_BITLEN(l,r)==len); return slice(_BITMAP(l,r,sl),_BITMAP(l,r,sr));}
-  const _bitslice operator()(int l, int r, int i) const 
+  const _bitslice operator()(bnd_t l, bnd_t r, bnd_t i) const 
               { ASSERT(_BITLEN(l,r)==len); return operator[](_BITMAP(l,r,i)); }
-  const _bitslice operator()(int l, int r, int sl, int sr) const 
+  const _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const 
     {ASSERT(_BITLEN(l,r)==len); return slice(_BITMAP(l,r,sl),_BITMAP(l,r,sr));}
                               
   // Implicit cast to working (rhs) bit vector representation
   operator const _bit() const                { const_cast<bit*>(this)->get();
-                      return _bit(const_cast<_bit::chunk*>(vec), len, usign); }
+                      return _bit(const_cast<_bit::chunk*>(this->vec), len, usign); }
   
   // Create a piece for a bit vector bus
-  virtual _bitbus_element* bus_slice(int sl, int sr) 
-                       { return new _bitbus_element(vec, len, usign, sl, sr); }
-  _bitbus_element* bus_slice(int l, int r, int sl, int sr) 
+  virtual _bitbus_element* bus_slice(len_t sl, len_t sr) 
+                       { return new _bitbus_element(this->vec, len, usign, sl, sr); }
+  _bitbus_element* bus_slice(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) 
                         { return bus_slice(_BITMAP(l,r,sl), _BITMAP(l,r,sr)); }
 
 protected:
-  // Convert to __bit base class
-  _bit ref(void)                              { return _bit(vec, len, usign); }
+  // Convert to _bit base class
+  _bit ref(void)                              { return _bit(this->vec, len, usign); }
   const _bit val(void) const                  { return operator const _bit(); }
   
   // Create a slice
-  virtual _bitslice slice(int l, int r, bool usgn = usign)
-                             { return _bitslice(vec, len, usign, l, r, usgn); }
-  virtual const _bitslice slice(int l, int r, bool usgn = usign) const
-   { return _bitslice(const_cast<_bit::chunk*>(vec), len, usign, l, r, usgn); }
+  virtual _bitslice slice(_bit::len_t l, _bit::len_t r, bool usgn = usign)
+                             { return _bitslice(this->vec, len, usign, l, r, usgn); }
+  virtual const _bitslice slice(len_t l, len_t r, bool usgn = usign) const
+   { return _bitslice(const_cast<_bit::chunk*>(this->vec), len, usign, l, r, usgn); }
   
   // Functions for synchronization to be overloaded by bit busses
   virtual bit& put()                                          { return *this; }
@@ -145,8 +155,12 @@ protected:
 
 /* --- Bit vector bus (concatenated bit slices) --- */
 
-template<int len, bool usign=false> class _bitbus: public bit<len,usign>
+template<_bit::len_t len, bool usign=false> class _bitbus
+  : public bit<len,usign>
 {
+  typedef _bit::len_t len_t;  
+  typedef _bit::bnd_t bnd_t;
+  
 protected:
   _bitbus_element* lst;
 
@@ -159,26 +173,29 @@ public:
      { for(_bitbus_element* el=lst; el; el=lst)  { lst=el->next; delete el; } }
   
   // Create a piece for a bit vector bus
-  virtual _bitbus_element* bus_slice(int sl, int sr) 
+  virtual _bitbus_element* bus_slice(len_t sl, len_t sr) 
                                                  { return lst->slice(sl, sr); }
 protected:  
   // Create a slice
-  virtual _bitslice slice(int l, int r, bool usgn = usign)
-                        { return _bitslice(vec, len, usign, l, r, usgn, lst); }
-  virtual const _bitslice slice(int l, int r, bool usgn = usign) const
-                            { return _bitslice(const_cast<_bit::chunk*>(vec), 
+  virtual _bitslice slice(len_t l, len_t r, bool usgn = usign)
+                        { return _bitslice(this->vec, len, usign, l, r, usgn, lst); }
+  virtual const _bitslice slice(len_t l, len_t r, bool usgn = usign) const
+                            { return _bitslice(const_cast<_bit::chunk*>(this->vec), 
                     		                len, usign, l, r, usgn, lst); }
   // Sync with original vectors
-  virtual _bitbus& put()                     { lst->put(ref()); return *this; }
-  virtual _bitbus& get()                     { lst->get(ref()); return *this; }
+  virtual _bitbus& put()                     { lst->put(this->ref()); return *this; }
+  virtual _bitbus& get()                     { lst->get(this->ref()); return *this; }
 };
 
 
 
 /* --- Shadow class for use inside unions --- */
 
-template<int len, bool usign=false> class _bit_: public _bitrep<len>
+template<_bit::len_t len, bool usign=false> class _bit_
 {
+  typedef _bit::len_t len_t;  
+  typedef _bit::bnd_t bnd_t;
+  
 public:  
   /* Cast to real bit vector
      Will fail miserably, cast from non-virtual to virtual class!
@@ -188,13 +205,15 @@ public:
   
   // Cast to working bit vector
   operator const _bit (void) const 
-                    { return _bit(const_cast<_bit::chunk*>(vec), len, usign); }
+               { return _bit(const_cast<_bit::chunk*>(data.vec), len, usign); }
   
   // Assignments
   _bit_& operator=(const _bit& c)                  { ref() = c; return *this; }
   _bit_& operator=(const _bitslice& s)             { ref() = s; return *this; }
-  template<int l, bool us> 
+  template<len_t l, bool us> 
     _bit_& operator=(const bit<l,us>& c)           { ref() = c; return *this; }
+  template<len_t l, bool us> 
+    _bit_& operator=(const _bit_<l,us>& c)         { ref() = c; return *this; }
   _bit_& operator=(const char* s)                  { ref() = s; return *this; }
   _bit_& operator=(int i)                          { ref() = i; return *this; }
   _bit_& operator=(unsigned int i)                 { ref() = i; return *this; }
@@ -218,8 +237,8 @@ public:
   _bit_& operator%=(const _bit& op)              { ref() %= op; return *this; }
 
   // Shift operators
-  _bit_& operator<<=(int sh)                    { ref() <<= sh; return *this; }
-  _bit_& operator>>=(int sh)                    { ref() >>= sh; return *this; }
+  _bit_& operator<<=(unsigned int sh)           { ref() <<= sh; return *this; }
+  _bit_& operator>>=(unsigned int sh)           { ref() >>= sh; return *this; }
 
   // Flip the bits in the vector
   _bit_& flip(void)                             { ref().flip(); return *this; }
@@ -244,35 +263,46 @@ public:
   long double toLDouble(void) const               { return val().toLDouble(); }
                                   
   // Slicing
-  _bitslice operator[](int i)                     { return slice(i, i, true); }
-  _bitslice operator()(int sl, int sr)                { return slice(sl, sr); }
-  const _bitslice operator[](int i) const         { return slice(i, i, true); }
-  const _bitslice operator()(int sl, int sr) const    { return slice(sl, sr); }
+  _bitslice operator[](len_t i)                   { return slice(i, i, true); }
+  _bitslice operator()(len_t sl, len_t sr)            { return slice(sl, sr); }
+  const _bitslice operator[](len_t i) const       { return slice(i, i, true); }
+  const _bitslice operator()(len_t sl, len_t sr) const {return slice(sl, sr); }
 
   // Mapped slicing 
-  _bitslice operator()(int l, int r, int i)    
+  _bitslice operator()(bnd_t l, bnd_t r, bnd_t i)    
               { ASSERT(_BITLEN(l,r)==len); return operator[](_BITMAP(l,r,i)); }
-  _bitslice operator()(int l, int r, int sl, int sr) 
+  _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) 
     {ASSERT(_BITLEN(l,r)==len); return slice(_BITMAP(l,r,sl),_BITMAP(l,r,sr));}
-  const _bitslice operator()(int l, int r, int i) const 
+  const _bitslice operator()(bnd_t l, bnd_t r, bnd_t i) const 
               { ASSERT(_BITLEN(l,r)==len); return operator[](_BITMAP(l,r,i)); }
-  const _bitslice operator()(int l, int r, int sl, int sr) const 
+  const _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const 
     {ASSERT(_BITLEN(l,r)==len); return slice(_BITMAP(l,r,sl),_BITMAP(l,r,sr));}
   
 protected:
-  // Convert to __bit base class
-  _bit ref(void)                              { return _bit(vec, len, usign); }
+  // Convert to _bit base class
+  _bit ref(void)                         { return _bit(data.vec, len, usign); }
   const _bit val(void) const                  { return operator const _bit(); }
 
   // Create a slice
-  _bitslice slice(int l, int r, bool usgn = usign) 
-                             { return _bitslice(vec, len, usign, l, r, usgn); }
-  const _bitslice slice(int l, int r, bool usgn = usign) const
-   { return _bitslice(const_cast<_bit::chunk*>(vec), len, usign, l, r, usgn); }
+  _bitslice slice(len_t l, len_t r, bool usgn = usign) 
+                        { return _bitslice(data.vec, len, usign, l, r, usgn); }
+  const _bitslice slice(len_t l, len_t r, bool usgn = usign) const 
+                      { return _bitslice(const_cast<_bit::chunk*>(data.vec),
+                                                     len, usign, l, r, usgn); }
 
 private:
+#ifdef GL_SYSTEM_VPTR_IS_FIRST_IN_CLASS
   // Simulate room needed for virtual method table pointer in real class
   void*  vptr;
+#endif
+  
+  // Data area
+  _bitrep<len> data;
+  
+#ifdef GL_SYSTEM_VPTR_IS_LAST_IN_CLASS
+  // Simulate room needed for virtual method table pointer in real class
+  void*  vptr;
+#endif
 };
 
 
@@ -284,13 +314,13 @@ private:
 /* Result of an operation */
 
 
-template<int len, bool usign> class _bitres: public _bit_<len,usign>
+template<_bit::len_t len, bool usign> class _bitres: public _bit_<len,usign>
 {
 public:
   // Constructors
   _bitres(const _bit& op1, char type) 
                                           { _bit(vec, len, usign, op1, type); }
-  _bitres(const _bit& op1, char type, int sh)
+  _bitres(const _bit& op1, char type, unsigned int sh)
                                       { _bit(vec, len, usign, op1, type, sh); }
   _bitres(const _bit& op1, char type, const _bit& op2)
                                      { _bit(vec, len, usign, op1, type, op2); }
@@ -312,79 +342,79 @@ public:
           _BITUSGN_BINARY(len1,usign1,len2,usign2)>
 
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const bit<len1,usign1>&
     operator+(const bit<len1,usign1>& op1) {
       return op1;
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
     operator~(const bit<len1,usign1>& op1) {
       return _BIT_UNARY(len1,usign1)(op1, '~');
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
     operator-(const bit<len1,usign1>& op1) {
       return _BIT_UNARY(len1,usign1)(op1, '-');
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
-    operator>>(const bit<len1,usign1>& op1, int sh) {
+    operator>>(const bit<len1,usign1>& op1, unsigned int sh) {
       return _BIT_UNARY(len1,usign1)(op1, '>', sh);
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
-    operator<<(const bit<len1,usign1>& op1, int sh) {
+    operator<<(const bit<len1,usign1>& op1, unsigned int sh) {
       return _BIT_UNARY(len1,usign1)(op1, '<', sh);
 }
   
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator+(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '+', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator-(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '-', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator*(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '*', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator/(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '/', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator%(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '%', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator&(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '&', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator|(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '|', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2>
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2>
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator^(const bit<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '^', op2);
@@ -395,159 +425,159 @@ template<int len1, bool usign1, int len2, bool usign2>
 /* ------------------------------------------------------------------------- */
 /* Operators with shadow class*/
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _bit_<len1,usign1>&
     operator+(const _bit_<len1,usign1>& op1) {
       return op1;
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
     operator~(const _bit_<len1,usign1>& op1) {
       return _BIT_UNARY(len1,usign1)(op1, '~');
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
     operator-(const _bit_<len1,usign1>& op1) {
       return _BIT_UNARY(len1,usign1)(op1, '-');
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
-    operator<<(const _bit_<len1,usign1>& op1, int sh) {
+    operator<<(const _bit_<len1,usign1>& op1, unsigned int sh) {
       return _BIT_UNARY(len1,usign1)(op1, '<', sh);
 }
 
-template<int len1, bool usign1>
+template<_bit::len_t len1, bool usign1>
   const _BIT_UNARY(len1,usign1)
-    operator>>(const _bit_<len1,usign1>& op1, int sh) {
+    operator>>(const _bit_<len1,usign1>& op1, unsigned int sh) {
       return _BIT_UNARY(len1,usign1)(op1, '>', sh);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator+(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '+', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator+(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '+', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator+(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '+', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator-(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '-', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator-(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '-', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator-(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '-', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator*(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '*', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator*(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '*', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator*(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '*', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator/(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '/', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator/(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '/', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator/(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '/', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator%(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '%', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator%(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '%', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator%(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '%', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator&(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '&', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator&(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '&', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator&(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '&', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator|(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '|', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator|(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '|', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator|(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '|', op2);
 }
 
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator^(const _bit_<len1,usign1>& op1, const bit<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '^', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator^(const bit<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '^', op2);
 }
-template<int len1, bool usign1, int len2, bool usign2> 
+template<_bit::len_t len1, bool usign1, _bit::len_t len2, bool usign2> 
   const _BIT_BINARY(len1,usign1,len2,usign2)
     operator^(const _bit_<len1,usign1>& op1, const _bit_<len2,usign2>& op2) {
       return _BIT_BINARY(len1,usign1,len2,usign2)(op1, '^', op2);
