@@ -2,6 +2,7 @@
 // longlong.h: "long long" data type support
 // ============================================================================
 // 
+// 04/08/02 <doemer>	added typedefs _LONGLONG and _ULONGLONG
 // 11/28/01 <gerstl>	Removed ambiguities, move more casting into scc
 // 05/24/01 <gerstl>	Initial version
 //
@@ -39,6 +40,9 @@
 #ifndef ULLONG_MAX
 #define ULLONG_MAX ULONG_LONG_MAX
 #endif
+/* --- Provide typedefs --- */
+typedef long long _LONGLONG;
+typedef unsigned long long _ULONGLONG;
 /* ---  Global functions --- */
 extern "C" {
 // conversion of long long to string (see sim.sh)  
@@ -52,7 +56,9 @@ unsigned long long str2ull(unsigned int base, const char* str);
 /* Non-native long long data type support */
 // Disable native long long on Solaris
 // Non-native long long version of library    
+// Provide constants
 /* --- Forward declaration of signed/unsigned class --- */
+/* --- Provide typedefs --- */
 /* --- Global functions --- */
 // conversion of long long to string (see sim.sh)  
 // conversion of string to long long (see sim.sh)
