@@ -2,6 +2,9 @@
 // sys/bit.h: Bit Vector Library, system level
 // ============================================================================
 //
+// 03/06/03 RD  made members of class __bitslice public so that they
+//              can be accessed by classes _specc::bufbus_elem and
+//              _specc::sigbus_elem
 // 11/28/01 <gerstl>	Removed constructors from shadow _longlong_
 // 11/17/01 <gerstl>	Added include/define for vptr from GL_System.h
 // 09/27/01 <gerstl>	Added constructor from unqualified char
@@ -366,7 +369,7 @@ protected:
 
 class __bitslice
 {
-protected:
+public:         // changed from protected: (03/06/03, RD)
   typedef _bit::len_t len_t;
   typedef _bit::bnd_t bnd_t;
   
